@@ -272,7 +272,7 @@ gst_audio_base_src_dispose (GObject * object)
 
   GST_OBJECT_LOCK (src);
   if (src->clock) {
-    gst_audio_clock_invalidate (src->clock);
+    gst_audio_clock_invalidate (GST_AUDIO_CLOCK (src->clock));
     gst_object_unref (src->clock);
     src->clock = NULL;
   }
@@ -583,6 +583,9 @@ gst_audio_base_src_setcaps (GstBaseSrc * bsrc, GstCaps * caps)
   g_object_notify (G_OBJECT (src), "actual-buffer-time");
   g_object_notify (G_OBJECT (src), "actual-latency-time");
 
+  gst_element_post_message (GST_ELEMENT_CAST (bsrc),
+      gst_message_new_latency (GST_OBJECT (bsrc)));
+
   return TRUE;
 
   /* ERRORS */
@@ -1024,7 +1027,7 @@ gst_audio_base_src_create (GstBaseSrc * bsrc, guint64 offset, guint length,
     } else {
       /* to get the timestamp against the clock we also need to add our
        * offset */
-      timestamp = gst_audio_clock_adjust (clock, timestamp);
+      timestamp = gst_audio_clock_adjust (GST_AUDIO_CLOCK (clock), timestamp);
     }
 
     /* we are not slaved, subtract base_time */
