@@ -20,12 +20,13 @@
  * Boston, MA 02110-1301, USA.
  */
 
+#ifndef __GST_AUDIO_AUDIO_H__
+#include <gst/audio/audio.h>
+#endif
+
 #ifndef __GST_AUDIO_RING_BUFFER_H__
 #define __GST_AUDIO_RING_BUFFER_H__
 
-#include <gst/gst.h>
-#include <gst/audio/audio.h>
-
 G_BEGIN_DECLS
 
 #define GST_TYPE_AUDIO_RING_BUFFER             (gst_audio_ring_buffer_get_type())
@@ -204,8 +205,10 @@ struct _GstAudioRingBuffer {
   gint                        may_start;
   gboolean                    active;
 
+  GDestroyNotify              cb_data_notify;
+
   /*< private >*/
-  gpointer _gst_reserved[GST_PADDING];
+  gpointer _gst_reserved[GST_PADDING - 1];
 };
 
 /**
@@ -219,7 +222,7 @@ struct _GstAudioRingBuffer {
  * @pause: pause processing of samples
  * @resume: resume processing of samples after pause
  * @stop: stop processing of samples
- * @delay: get number of samples queued in device
+ * @delay: get number of frames queued in device
  * @activate: activate the thread that starts pulling and monitoring the
  * consumed segments in the device.
  * @commit: write samples into the ringbuffer
@@ -259,9 +262,13 @@ struct _GstAudioRingBufferClass {
 GType gst_audio_ring_buffer_get_type(void);
 
 /* callback stuff */
-void            gst_audio_ring_buffer_set_callback    (GstAudioRingBuffer *buf,
-                                                       GstAudioRingBufferCallback cb,
-                                                       gpointer user_data);
+void            gst_audio_ring_buffer_set_callback      (GstAudioRingBuffer *buf,
+                                                         GstAudioRingBufferCallback cb,
+                                                         gpointer user_data);
+void            gst_audio_ring_buffer_set_callback_full (GstAudioRingBuffer *buf,
+                                                         GstAudioRingBufferCallback cb,
+                                                         gpointer user_data,
+                                                         GDestroyNotify notify);
 
 gboolean        gst_audio_ring_buffer_parse_caps      (GstAudioRingBufferSpec *spec, GstCaps *caps);
 void            gst_audio_ring_buffer_debug_spec_caps (GstAudioRingBufferSpec *spec);
@@ -332,6 +339,10 @@ void            gst_audio_ring_buffer_advance         (GstAudioRingBuffer *buf,
 
 void            gst_audio_ring_buffer_may_start       (GstAudioRingBuffer *buf, gboolean allowed);
 
+#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstAudioRingBuffer, gst_object_unref)
+#endif
+
 G_END_DECLS
 
 #endif /* __GST_AUDIO_RING_BUFFER_H__ */
