@@ -47,6 +47,8 @@
 #include <gst/video/video.h>
 #include <gst/audio/audio.h>
 
+#include <string.h>
+
 #include "pbutils.h"
 #include "pbutils-private.h"
 
@@ -55,6 +57,7 @@ GST_DEBUG_CATEGORY_STATIC (discoverer_debug);
 
 static GQuark _CAPS_QUARK;
 static GQuark _TAGS_QUARK;
+static GQuark _ELEMENT_SRCPAD_QUARK;
 static GQuark _TOC_QUARK;
 static GQuark _STREAM_ID_QUARK;
 static GQuark _MISSING_PLUGIN_QUARK;
@@ -137,6 +140,7 @@ _do_init (void)
   GST_DEBUG_CATEGORY_INIT (discoverer_debug, "discoverer", 0, "Discoverer");
 
   _CAPS_QUARK = g_quark_from_static_string ("caps");
+  _ELEMENT_SRCPAD_QUARK = g_quark_from_static_string ("element-srcpad");
   _TAGS_QUARK = g_quark_from_static_string ("tags");
   _TOC_QUARK = g_quark_from_static_string ("toc");
   _STREAM_ID_QUARK = g_quark_from_static_string ("stream-id");
@@ -393,7 +397,8 @@ gst_discoverer_dispose (GObject * obj)
 
     /* pipeline was set to NULL in _reset */
     gst_object_unref (dc->priv->pipeline);
-    gst_object_unref (dc->priv->bus);
+    if (dc->priv->bus)
+      gst_object_unref (dc->priv->bus);
 
     dc->priv->pipeline = NULL;
     dc->priv->uridecodebin = NULL;
@@ -547,7 +552,9 @@ got_subtitle_data (GstPad * pad, GstPadProbeInfo * info, GstDiscoverer * dc)
 {
 
   if (!(GST_IS_BUFFER (info->data) || (GST_IS_EVENT (info->data)
-              && GST_EVENT_TYPE ((GstEvent *) info->data) == GST_EVENT_GAP)))
+              && (GST_EVENT_TYPE ((GstEvent *) info->data) == GST_EVENT_GAP
+                  || GST_EVENT_TYPE ((GstEvent *) info->data) ==
+                  GST_EVENT_EOS))))
     return GST_PAD_PROBE_OK;
 
 
@@ -799,19 +806,36 @@ static GstDiscovererStreamInfo *
 collect_information (GstDiscoverer * dc, const GstStructure * st,
     GstDiscovererStreamInfo * parent)
 {
-  GstCaps *caps;
+  GstPad *srcpad;
+  GstCaps *caps = NULL;
   GstStructure *caps_st;
   GstTagList *tags_st;
   const gchar *name;
-  int tmp;
+  gint tmp, tmp2;
   guint utmp;
 
-  if (!st || !gst_structure_id_has_field (st, _CAPS_QUARK)) {
+  if (!st || (!gst_structure_id_has_field (st, _CAPS_QUARK)
+          && !gst_structure_id_has_field (st, _ELEMENT_SRCPAD_QUARK))) {
     GST_WARNING ("Couldn't find caps !");
     return make_info (parent, GST_TYPE_DISCOVERER_STREAM_INFO, NULL);
   }
 
-  gst_structure_id_get (st, _CAPS_QUARK, GST_TYPE_CAPS, &caps, NULL);
+  if (gst_structure_id_get (st, _ELEMENT_SRCPAD_QUARK, GST_TYPE_PAD, &srcpad,
+          NULL)) {
+    caps = gst_pad_get_current_caps (srcpad);
+    gst_object_unref (srcpad);
+  }
+  if (!caps) {
+    gst_structure_id_get (st, _CAPS_QUARK, GST_TYPE_CAPS, &caps, NULL);
+  }
+
+  if (!caps || gst_caps_is_empty (caps) || gst_caps_is_any (caps)) {
+    GST_WARNING ("Couldn't find caps !");
+    if (caps)
+      gst_caps_unref (caps);
+    return make_info (parent, GST_TYPE_DISCOVERER_STREAM_INFO, NULL);
+  }
+
   caps_st = gst_caps_get_structure (caps, 0);
   name = gst_structure_get_name (caps_st);
 
@@ -837,7 +861,8 @@ collect_information (GstDiscoverer * dc, const GstStructure * st,
 
       format = gst_audio_format_from_string (format_str);
       finfo = gst_audio_format_get_info (format);
-      info->depth = GST_AUDIO_FORMAT_INFO_DEPTH (finfo);
+      if (finfo)
+        info->depth = GST_AUDIO_FORMAT_INFO_DEPTH (finfo);
     }
 
     if (gst_structure_id_has_field (st, _TAGS_QUARK)) {
@@ -869,27 +894,51 @@ collect_information (GstDiscoverer * dc, const GstStructure * st,
   } else if (g_str_has_prefix (name, "video/") ||
       g_str_has_prefix (name, "image/")) {
     GstDiscovererVideoInfo *info;
-    GstVideoInfo vinfo;
+    const gchar *caps_str;
 
     info = (GstDiscovererVideoInfo *) make_info (parent,
         GST_TYPE_DISCOVERER_VIDEO_INFO, caps);
 
-    if (gst_video_info_from_caps (&vinfo, caps)) {
-      info->width = (guint) vinfo.width;
-      info->height = (guint) vinfo.height;
+    if (gst_structure_get_int (caps_st, "width", &tmp))
+      info->width = (guint) tmp;
+    if (gst_structure_get_int (caps_st, "height", &tmp))
+      info->height = (guint) tmp;
 
-      info->depth = vinfo.finfo->bits * vinfo.finfo->n_components;
-
-      info->par_num = vinfo.par_n;
-      info->par_denom = vinfo.par_d;
-
-      info->framerate_num = vinfo.fps_n;
-      info->framerate_denom = vinfo.fps_d;
-
-      info->interlaced =
-          vinfo.interlace_mode != GST_VIDEO_INTERLACE_MODE_PROGRESSIVE;
+    if (gst_structure_get_fraction (caps_st, "framerate", &tmp, &tmp2)) {
+      info->framerate_num = (guint) tmp;
+      info->framerate_denom = (guint) tmp2;
+    } else {
+      info->framerate_num = 0;
+      info->framerate_denom = 1;
     }
 
+    if (gst_structure_get_fraction (caps_st, "pixel-aspect-ratio", &tmp, &tmp2)) {
+      info->par_num = (guint) tmp;
+      info->par_denom = (guint) tmp2;
+    } else {
+      info->par_num = 1;
+      info->par_denom = 1;
+    }
+
+    /* FIXME: we only want to extract depth if raw video is what's in the
+     * container (i.e. not if there is a decoder involved) */
+    caps_str = gst_structure_get_string (caps_st, "format");
+    if (caps_str != NULL) {
+      const GstVideoFormatInfo *finfo;
+      GstVideoFormat format;
+
+      format = gst_video_format_from_string (caps_str);
+      finfo = gst_video_format_get_info (format);
+      if (finfo)
+        info->depth = finfo->bits * finfo->n_components;
+    }
+
+    caps_str = gst_structure_get_string (caps_st, "interlace-mode");
+    if (!caps_str || strcmp (caps_str, "progressive") == 0)
+      info->interlaced = FALSE;
+    else
+      info->interlaced = TRUE;
+
     if (gst_structure_id_has_field (st, _TAGS_QUARK)) {
       gst_structure_id_get (st, _TAGS_QUARK, GST_TYPE_TAG_LIST, &tags_st, NULL);
       if (gst_tag_list_get_uint (tags_st, GST_TAG_BITRATE, &utmp) ||
@@ -1004,10 +1053,20 @@ find_stream_for_node (GstDiscoverer * dc, const GstStructure * topology)
 static gboolean
 child_is_same_stream (const GstCaps * _parent, const GstCaps * child)
 {
-  GstCaps *parent = gst_caps_copy (_parent);
-  guint i, size = gst_caps_get_size (parent);
+  GstCaps *parent;
+  guint i, size;
   gboolean res;
 
+  if (_parent == child)
+    return TRUE;
+  if (!_parent)
+    return FALSE;
+  if (!child)
+    return FALSE;
+
+  parent = gst_caps_copy (_parent);
+  size = gst_caps_get_size (parent);
+
   for (i = 0; i < size; i++) {
     gst_structure_remove_field (gst_caps_get_structure (parent, i), "parsed");
     gst_structure_remove_field (gst_caps_get_structure (parent, i), "framed");
@@ -1024,6 +1083,13 @@ child_is_raw_stream (const GstCaps * parent, const GstCaps * child)
   const GstStructure *st1, *st2;
   const gchar *name1, *name2;
 
+  if (parent == child)
+    return TRUE;
+  if (!parent)
+    return FALSE;
+  if (!child)
+    return FALSE;
+
   st1 = gst_caps_get_structure (parent, 0);
   name1 = gst_structure_get_name (st1);
   st2 = gst_caps_get_structure (child, 0);
@@ -1075,6 +1141,8 @@ parse_stream_topology (GstDiscoverer * dc, const GstStructure * topology,
       /* FIXME : aggregate with information from main streams */
       GST_DEBUG ("Coudn't find 'next' ! might be the last entry");
     } else {
+      GstPad *srcpad;
+
       st = (GstStructure *) gst_value_get_structure (nval);
 
       GST_DEBUG ("next is a structure %" GST_PTR_FORMAT, st);
@@ -1082,7 +1150,16 @@ parse_stream_topology (GstDiscoverer * dc, const GstStructure * topology,
       if (!parent)
         parent = res;
 
-      if (gst_structure_id_get (st, _CAPS_QUARK, GST_TYPE_CAPS, &caps, NULL)) {
+      if (gst_structure_id_get (st, _ELEMENT_SRCPAD_QUARK, GST_TYPE_PAD,
+              &srcpad, NULL)) {
+        caps = gst_pad_get_current_caps (srcpad);
+        gst_object_unref (srcpad);
+      }
+      if (!caps) {
+        gst_structure_id_get (st, _CAPS_QUARK, GST_TYPE_CAPS, &caps, NULL);
+      }
+
+      if (caps) {
         if (child_is_same_stream (parent->caps, caps)) {
           /* We sometimes get an extra sub-stream from the parser. If this is
            * the case, we just replace the parent caps with this stream's caps
@@ -1116,9 +1193,18 @@ parse_stream_topology (GstDiscoverer * dc, const GstStructure * topology,
     guint i, len;
     GstDiscovererContainerInfo *cont;
     GstTagList *tags;
+    GstPad *srcpad;
 
-    if (!gst_structure_id_get (topology, _CAPS_QUARK,
-            GST_TYPE_CAPS, &caps, NULL))
+    if (gst_structure_id_get (topology, _ELEMENT_SRCPAD_QUARK, GST_TYPE_PAD,
+            &srcpad, NULL)) {
+      caps = gst_pad_get_current_caps (srcpad);
+      gst_object_unref (srcpad);
+    }
+    if (!caps) {
+      gst_structure_id_get (topology, _CAPS_QUARK, GST_TYPE_CAPS, &caps, NULL);
+    }
+
+    if (!caps)
       GST_WARNING ("Couldn't find caps !");
 
     len = gst_value_list_get_size (nval);
@@ -1307,6 +1393,7 @@ static gboolean
 handle_message (GstDiscoverer * dc, GstMessage * msg)
 {
   gboolean done = FALSE;
+  const gchar *dump_name = NULL;
 
   GST_DEBUG_OBJECT (GST_MESSAGE_SRC (msg), "got a %s message",
       GST_MESSAGE_TYPE_NAME (msg));
@@ -1324,6 +1411,7 @@ handle_message (GstDiscoverer * dc, GstMessage * msg)
 
       /* We need to stop */
       done = TRUE;
+      dump_name = "gst-discoverer-error";
 
       /* Don't override missing plugin result code for missing plugin errors */
       if (dc->priv->current_info->result != GST_DISCOVERER_MISSING_PLUGINS ||
@@ -1337,9 +1425,23 @@ handle_message (GstDiscoverer * dc, GstMessage * msg)
     }
       break;
 
+    case GST_MESSAGE_WARNING:{
+      GError *err;
+      gchar *debug = NULL;
+
+      gst_message_parse_warning (msg, &err, &debug);
+      GST_WARNING_OBJECT (GST_MESSAGE_SRC (msg),
+          "Got a warning [debug:%s], [message:%s]", debug, err->message);
+      g_clear_error (&err);
+      g_free (debug);
+      dump_name = "gst-discoverer-warning";
+      break;
+    }
+
     case GST_MESSAGE_EOS:
       GST_DEBUG ("Got EOS !");
       done = TRUE;
+      dump_name = "gst-discoverer-eos";
       break;
 
     case GST_MESSAGE_APPLICATION:{
@@ -1350,8 +1452,10 @@ handle_message (GstDiscoverer * dc, GstMessage * msg)
       DISCO_LOCK (dc);
       async_done = dc->priv->async_done;
       DISCO_UNLOCK (dc);
-      if (g_str_equal (name, "DiscovererDone") && async_done)
-        return TRUE;
+      if (g_str_equal (name, "DiscovererDone") && async_done) {
+        done = TRUE;
+        dump_name = "gst-discoverer-async-done-subtitle";
+      }
       break;
     }
 
@@ -1361,6 +1465,7 @@ handle_message (GstDiscoverer * dc, GstMessage * msg)
         DISCO_LOCK (dc);
         if (dc->priv->pending_subtitle_pads == 0) {
           done = TRUE;
+          dump_name = "gst-discoverer-async-done";
         } else {
           /* Remember that ASYNC_DONE has been received, wait for subtitles */
           dc->priv->async_done = TRUE;
@@ -1436,6 +1541,11 @@ handle_message (GstDiscoverer * dc, GstMessage * msg)
       break;
   }
 
+  if (dump_name != NULL) {
+    /* dump graph when done or for warnings */
+    GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS (GST_BIN (dc->priv->pipeline),
+        GST_DEBUG_GRAPH_SHOW_ALL, dump_name);
+  }
   return done;
 }
 
