@@ -611,11 +611,16 @@ gst_rtp_base_depayload_handle_event (GstRTPBaseDepayload * filter,
     }
     case GST_EVENT_SEGMENT:
     {
+      GstSegment segment;
+
       GST_OBJECT_LOCK (filter);
-      gst_event_copy_segment (event, &filter->segment);
-      if (filter->segment.format != GST_FORMAT_TIME)
-        GST_ERROR_OBJECT (filter,
-            "Non-TIME segments are not supported and will likely fail");
+      gst_event_copy_segment (event, &segment);
+
+      if (segment.format != GST_FORMAT_TIME) {
+        GST_ERROR_OBJECT (filter, "Segment with non-TIME format not supported");
+        res = FALSE;
+      }
+      filter->segment = segment;
       GST_OBJECT_UNLOCK (filter);
 
       /* don't pass the event downstream, we generate our own segment including
@@ -720,11 +725,8 @@ create_segment_event (GstRTPBaseDepayload * filter, guint rtptime,
   if (position == -1)
     position = start;
 
-  if (G_LIKELY (filter->segment.format == GST_FORMAT_TIME))
-    running_time = gst_segment_to_running_time (&filter->segment,
-        GST_FORMAT_TIME, start);
-  else
-    running_time = 0;
+  running_time = gst_segment_to_running_time (&filter->segment,
+      GST_FORMAT_TIME, start);
 
   gst_segment_init (&segment, GST_FORMAT_TIME);
   segment.rate = priv->play_speed;
@@ -742,16 +744,9 @@ create_segment_event (GstRTPBaseDepayload * filter, guint rtptime,
   return event;
 }
 
-typedef struct
-{
-  GstRTPBaseDepayload *depayload;
-  GstRTPBaseDepayloadClass *bclass;
-} HeaderData;
-
 static gboolean
-set_headers (GstBuffer ** buffer, guint idx, HeaderData * data)
+set_headers (GstBuffer ** buffer, guint idx, GstRTPBaseDepayload * depayload)
 {
-  GstRTPBaseDepayload *depayload = data->depayload;
   GstRTPBaseDepayloadPrivate *priv = depayload->priv;
   GstClockTime pts, dts, duration;
 
@@ -788,17 +783,12 @@ static GstFlowReturn
 gst_rtp_base_depayload_prepare_push (GstRTPBaseDepayload * filter,
     gboolean is_list, gpointer obj)
 {
-  HeaderData data;
-
-  data.depayload = filter;
-  data.bclass = GST_RTP_BASE_DEPAYLOAD_GET_CLASS (filter);
-
   if (is_list) {
     GstBufferList **blist = obj;
-    gst_buffer_list_foreach (*blist, (GstBufferListFunc) set_headers, &data);
+    gst_buffer_list_foreach (*blist, (GstBufferListFunc) set_headers, filter);
   } else {
     GstBuffer **buf = obj;
-    set_headers (buf, 0, &data);
+    set_headers (buf, 0, filter);
   }
 
   /* if this is the first buffer send a NEWSEGMENT */
