@@ -186,7 +186,9 @@ video_buffer_pool_set_config (GstBufferPool * pool, GstStructure * config)
       priv->video_align.stride_align[n] = max_align;
 
     /* apply the alignment to the info */
-    gst_video_info_align (&info, &priv->video_align);
+    if (!gst_video_info_align (&info, &priv->video_align))
+      goto failed_to_align;
+
     gst_buffer_pool_config_set_video_alignment (config, &priv->video_align);
 
     if (priv->params.align < max_align) {
@@ -197,8 +199,8 @@ video_buffer_pool_set_config (GstBufferPool * pool, GstStructure * config)
       gst_buffer_pool_config_set_allocator (config, allocator, &priv->params);
     }
   }
-  priv->info = info;
   info.size = MAX (size, info.size);
+  priv->info = info;
 
   gst_buffer_pool_config_set_params (config, caps, info.size, min_buffers,
       max_buffers);
@@ -227,7 +229,11 @@ wrong_size:
     GST_WARNING_OBJECT (pool,
         "Provided size is to small for the caps: %u", size);
     return FALSE;
-
+  }
+failed_to_align:
+  {
+    GST_WARNING_OBJECT (pool, "Failed to align");
+    return FALSE;
   }
 }
 
