@@ -315,7 +315,8 @@ static gboolean gst_encode_bin_setup_profile (GstEncodeBin * ebin,
     GstEncodingProfile * profile);
 
 static StreamGroup *_create_stream_group (GstEncodeBin * ebin,
-    GstEncodingProfile * sprof, const gchar * sinkpadname, GstCaps * sinkcaps);
+    GstEncodingProfile * sprof, const gchar * sinkpadname, GstCaps * sinkcaps,
+    gboolean * encoder_not_found);
 static void stream_group_remove (GstEncodeBin * ebin, StreamGroup * sgroup);
 static void stream_group_free (GstEncodeBin * ebin, StreamGroup * sgroup);
 static GstPad *gst_encode_bin_request_pad_signal (GstEncodeBin * encodebin,
@@ -325,6 +326,8 @@ static GstPad *gst_encode_bin_request_profile_pad_signal (GstEncodeBin *
 
 static inline GstElement *_get_formatter (GstEncodeBin * ebin,
     GstEncodingProfile * sprof);
+static void _post_missing_plugin_message (GstEncodeBin * ebin,
+    GstEncodingProfile * prof);
 
 static void
 gst_encode_bin_class_init (GstEncodeBinClass * klass)
@@ -632,7 +635,7 @@ stream_profile_used_count (GstEncodeBin * ebin, GstEncodingProfile * sprof)
 
 static inline GstEncodingProfile *
 next_unused_stream_profile (GstEncodeBin * ebin, GType ptype,
-    const gchar * name, GstCaps * caps)
+    const gchar * name, GstCaps * caps, GstEncodingProfile * previous_profile)
 {
   GST_DEBUG_OBJECT (ebin, "ptype:%s, caps:%" GST_PTR_FORMAT,
       g_type_name (ptype), caps);
@@ -701,8 +704,11 @@ next_unused_stream_profile (GstEncodeBin * ebin, GType ptype,
         if (!gst_encoding_profile_is_enabled (sprof)) {
           GST_INFO_OBJECT (ebin, "%p is disabled, not using it", sprof);
         } else if (presence == 0
-            || (presence > stream_profile_used_count (ebin, sprof)))
-          return sprof;
+            || (presence > stream_profile_used_count (ebin, sprof))) {
+
+          if (sprof != previous_profile)
+            return sprof;
+        }
       } else if (caps && ptype == G_TYPE_NONE) {
         GstCaps *outcaps;
         gboolean res;
@@ -713,7 +719,7 @@ next_unused_stream_profile (GstEncodeBin * ebin, GType ptype,
         res = gst_caps_can_intersect (outcaps, caps);
         gst_caps_unref (outcaps);
 
-        if (res)
+        if (res && sprof != previous_profile)
           return sprof;
       }
     }
@@ -726,20 +732,39 @@ static GstPad *
 request_pad_for_stream (GstEncodeBin * encodebin, GType ptype,
     const gchar * name, GstCaps * caps)
 {
-  StreamGroup *sgroup;
-  GstEncodingProfile *sprof;
+  StreamGroup *sgroup = NULL;
+  GList *not_found_encoder_profs = NULL, *tmp;
+  GstEncodingProfile *sprof = NULL;
 
   GST_DEBUG_OBJECT (encodebin, "name:%s caps:%" GST_PTR_FORMAT, name, caps);
 
-  /* Figure out if we have a unused GstEncodingProfile we can use for
-   * these caps */
-  sprof = next_unused_stream_profile (encodebin, ptype, name, caps);
+  while (sgroup == NULL) {
+    gboolean encoder_not_found = FALSE;
+    /* Figure out if we have a unused GstEncodingProfile we can use for
+     * these caps */
+    sprof = next_unused_stream_profile (encodebin, ptype, name, caps, sprof);
 
-  if (G_UNLIKELY (sprof == NULL))
-    goto no_stream_profile;
+    if (G_UNLIKELY (sprof == NULL))
+      goto no_stream_profile;
 
-  sgroup = _create_stream_group (encodebin, sprof, name, caps);
-  if (G_UNLIKELY (sgroup == NULL))
+    sgroup = _create_stream_group (encodebin, sprof, name, caps,
+        &encoder_not_found);
+
+    if (G_UNLIKELY (sgroup))
+      break;
+
+    if (encoder_not_found) {
+      not_found_encoder_profs = g_list_prepend (not_found_encoder_profs, sprof);
+      if (name) {
+        GST_DEBUG ("Could not create an encoder for %s", name);
+        goto no_stream_group;
+      }
+    } else {
+      break;
+    }
+  }
+
+  if (!sgroup)
     goto no_stream_group;
 
   return sgroup->ghostpad;
@@ -752,6 +777,10 @@ no_stream_profile:
 
 no_stream_group:
   {
+    for (tmp = not_found_encoder_profs; tmp; tmp = tmp->next)
+      _post_missing_plugin_message (encodebin, tmp->data);
+    g_list_free (not_found_encoder_profs);
+
     GST_WARNING_OBJECT (encodebin, "Couldn't create a StreamGroup");
     return NULL;
   }
@@ -1140,8 +1169,12 @@ _post_missing_plugin_message (GstEncodeBin * ebin, GstEncodingProfile * prof)
   GstCaps *format;
   format = gst_encoding_profile_get_format (prof);
 
-  GST_ERROR_OBJECT (ebin, "Couldn't create encoder for format %" GST_PTR_FORMAT,
-      format);
+  GST_ERROR_OBJECT (ebin,
+      "Couldn't create encoder with preset %s and preset name %s"
+      " for format %" GST_PTR_FORMAT,
+      GST_STR_NULL (gst_encoding_profile_get_preset (prof)),
+      GST_STR_NULL (gst_encoding_profile_get_preset_name (prof)), format);
+
   /* missing plugin support */
   gst_element_post_message (GST_ELEMENT_CAST (ebin),
       gst_missing_encoder_message_new (GST_ELEMENT_CAST (ebin), format));
@@ -1178,10 +1211,13 @@ _set_up_fake_encoder_pad_probe (GstEncodeBin * ebin, StreamGroup * sgroup)
  * Create the elements, StreamGroup, add the sink pad, link it to the muxer
  *
  * sinkpadname: If non-NULL, that name will be assigned to the sink ghost pad
- * sinkcaps: If non-NULL will be used to figure out how to setup the group */
+ * sinkcaps: If non-NULL will be used to figure out how to setup the group
+ * encoder_not_found: If non NULL, set to TRUE if failure happened because
+ * the encoder could not be found
+ */
 static StreamGroup *
 _create_stream_group (GstEncodeBin * ebin, GstEncodingProfile * sprof,
-    const gchar * sinkpadname, GstCaps * sinkcaps)
+    const gchar * sinkpadname, GstCaps * sinkcaps, gboolean * encoder_not_found)
 {
   StreamGroup *sgroup = NULL;
   GstPad *sinkpad, *srcpad, *muxerpad = NULL;
@@ -1204,7 +1240,7 @@ _create_stream_group (GstEncodeBin * ebin, GstEncodingProfile * sprof,
   sgroup->profile = sprof;
 
   /* NOTE for people reading this code:
-   * 
+   *
    * We construct the group starting by the furthest downstream element
    * and making our way up adding/syncing/linking as we go.
    *
@@ -1224,11 +1260,11 @@ _create_stream_group (GstEncodeBin * ebin, GstEncodingProfile * sprof,
   }
 
   /* Output Queue.
-   * We only use a 1buffer long queue here, the actual queueing will be done
-   * in the input queue */
+   * The actual queueing will be done in the input queue, but some queuing
+   * after the encoder can be beneficial for encoding performance. */
   last = sgroup->outqueue = gst_element_factory_make ("queue", NULL);
-  g_object_set (sgroup->outqueue, "max-size-buffers", (guint32) 1,
-      "max-size-bytes", (guint32) 0, "max-size-time", (guint64) 0,
+  g_object_set (sgroup->outqueue, "max-size-buffers", (guint) 0,
+      "max-size-bytes", (guint) 0, "max-size-time", (guint64) 3 * GST_SECOND,
       "silent", TRUE, NULL);
 
   gst_bin_add (GST_BIN (ebin), sgroup->outqueue);
@@ -1306,8 +1342,8 @@ _create_stream_group (GstEncodeBin * ebin, GstEncodingProfile * sprof,
    * FIXME : figure out what max-size to use for the input queue */
   sgroup->inqueue = gst_element_factory_make ("queue", NULL);
   g_object_set (sgroup->inqueue, "max-size-buffers",
-      (guint32) ebin->queue_buffers_max, "max-size-bytes",
-      (guint32) ebin->queue_bytes_max, "max-size-time",
+      (guint) ebin->queue_buffers_max, "max-size-bytes",
+      (guint) ebin->queue_bytes_max, "max-size-time",
       (guint64) ebin->queue_time_max, "silent", TRUE, NULL);
 
   gst_bin_add (GST_BIN (ebin), sgroup->inqueue);
@@ -1393,7 +1429,11 @@ _create_stream_group (GstEncodeBin * ebin, GstEncodingProfile * sprof,
     g_object_unref (srcpad);
   } else if (gst_encoding_profile_get_preset (sgroup->profile)
       || gst_encoding_profile_get_preset_name (sgroup->profile)) {
-    _post_missing_plugin_message (ebin, sprof);
+
+    if (!encoder_not_found)
+      _post_missing_plugin_message (ebin, sprof);
+    else
+      *encoder_not_found = TRUE;
     goto cleanup;
   } else {
     /* passthrough can still work, if we discover that *
@@ -1881,7 +1921,7 @@ create_elements_and_pads (GstEncodeBin * ebin)
 
     /* 2. Ghost the muxer source pad */
 
-    /* FIXME : We should figure out if it's a static/request/dyamic pad, 
+    /* FIXME : We should figure out if it's a static/request/dyamic pad,
      * but for the time being let's assume it's a static pad :) */
     muxerpad = gst_element_get_static_pad (muxer, "src");
     if (G_UNLIKELY (muxerpad == NULL))
@@ -1903,14 +1943,15 @@ create_elements_and_pads (GstEncodeBin * ebin)
 
       if (gst_encoding_profile_get_presence (sprof) != 0 &&
           gst_encoding_profile_is_enabled (sprof)) {
-        if (G_UNLIKELY (_create_stream_group (ebin, sprof, NULL, NULL) == NULL))
+        if (G_UNLIKELY (_create_stream_group (ebin, sprof, NULL, NULL,
+                    NULL) == NULL))
           goto stream_error;
       }
     }
     gst_element_sync_state_with_parent (muxer);
   } else {
     if (G_UNLIKELY (_create_stream_group (ebin, ebin->profile, NULL,
-                NULL) == NULL))
+                NULL, NULL) == NULL))
       goto stream_error;
   }
 
@@ -1991,23 +2032,24 @@ stream_group_free (GstEncodeBin * ebin, StreamGroup * sgroup)
   if (sgroup->restriction_sid != 0)
     g_signal_handler_disconnect (sgroup->profile, sgroup->restriction_sid);
 
-  if (ebin->muxer) {
-    /* outqueue - Muxer */
-    tmppad = gst_element_get_static_pad (sgroup->outqueue, "src");
-    pad = gst_pad_get_peer (tmppad);
+  if (sgroup->outqueue) {
+    if (ebin->muxer) {
+      /* outqueue - Muxer */
+      tmppad = gst_element_get_static_pad (sgroup->outqueue, "src");
+      pad = gst_pad_get_peer (tmppad);
 
-    if (pad) {
-      /* Remove muxer request sink pad */
-      gst_pad_unlink (tmppad, pad);
-      if (GST_PAD_TEMPLATE_PRESENCE (GST_PAD_PAD_TEMPLATE (pad)) ==
-          GST_PAD_REQUEST)
-        gst_element_release_request_pad (ebin->muxer, pad);
-      gst_object_unref (pad);
+      if (pad) {
+        /* Remove muxer request sink pad */
+        gst_pad_unlink (tmppad, pad);
+        if (GST_PAD_TEMPLATE_PRESENCE (GST_PAD_PAD_TEMPLATE (pad)) ==
+            GST_PAD_REQUEST)
+          gst_element_release_request_pad (ebin->muxer, pad);
+        gst_object_unref (pad);
+      }
+      gst_object_unref (tmppad);
     }
-    gst_object_unref (tmppad);
-  }
-  if (sgroup->outqueue)
     gst_element_set_state (sgroup->outqueue, GST_STATE_NULL);
+  }
 
   if (sgroup->formatter) {
     /* capsfilter - formatter - outqueue */
@@ -2015,13 +2057,16 @@ stream_group_free (GstEncodeBin * ebin, StreamGroup * sgroup)
     gst_element_set_state (sgroup->outfilter, GST_STATE_NULL);
     gst_element_unlink (sgroup->formatter, sgroup->outqueue);
     gst_element_unlink (sgroup->outfilter, sgroup->formatter);
-  } else {
+  } else if (sgroup->outfilter) {
     /* Capsfilter - outqueue */
     gst_element_set_state (sgroup->outfilter, GST_STATE_NULL);
     gst_element_unlink (sgroup->outfilter, sgroup->outqueue);
   }
-  gst_element_set_state (sgroup->outqueue, GST_STATE_NULL);
-  gst_bin_remove (GST_BIN (ebin), sgroup->outqueue);
+
+  if (sgroup->outqueue) {
+    gst_element_set_state (sgroup->outqueue, GST_STATE_NULL);
+    gst_bin_remove (GST_BIN (ebin), sgroup->outqueue);
+  }
 
   /* streamcombiner - parser - capsfilter */
   if (sgroup->parser) {
