@@ -61,9 +61,13 @@ G_BEGIN_DECLS
  * @GST_VIDEO_TEST_SRC_CHROMA_ZONE_PLATE: Chroma zone plate
  * @GST_VIDEO_TEST_SRC_BALL: Moving ball
  * @GST_VIDEO_TEST_SRC_SMPTE100: SMPTE test pattern (100% color bars)
+ * @GST_VIDEO_TEST_SRC_SOLID: A solid color, defined by the
+ * #GstVideoTestSrc:foreground-color property
  * @GST_VIDEO_TEST_SRC_BAR: Bar with foreground color
  * @GST_VIDEO_TEST_SRC_PINWHEEL: Pinwheel
  * @GST_VIDEO_TEST_SRC_SPOKES: Spokes
+ * @GST_VIDEO_TEST_SRC_GRADIENT: Gradient
+ * @GST_VIDEO_TEST_SRC_COLORS: All colors
  *
  * The test pattern to produce.
  *
@@ -107,7 +111,9 @@ typedef enum {
   GST_VIDEO_TEST_SRC_SMPTE100,
   GST_VIDEO_TEST_SRC_BAR,
   GST_VIDEO_TEST_SRC_PINWHEEL,
-  GST_VIDEO_TEST_SRC_SPOKES
+  GST_VIDEO_TEST_SRC_SPOKES,
+  GST_VIDEO_TEST_SRC_GRADIENT,
+  GST_VIDEO_TEST_SRC_COLORS
 } GstVideoTestSrcPattern;
 
 typedef struct _GstVideoTestSrc GstVideoTestSrc;
@@ -134,11 +140,13 @@ struct _GstVideoTestSrc {
   gint y_invert;
 
   /* private */
+  /* FIXME 2.0: Change type to GstClockTime */
   gint64 timestamp_offset;              /* base offset */
 
   /* running time and frames for current caps */
   GstClockTime running_time;            /* total running time */
   gint64 n_frames;                      /* total frames sent */
+  gboolean reverse;
 
   /* previous caps running time and frames */
   GstClockTime accum_rtime;              /* accumulated running_time */
@@ -166,6 +174,9 @@ struct _GstVideoTestSrc {
   gint horizontal_offset;
   gint horizontal_speed;
 
+  /* smpte & snow */
+  guint random_state;
+
   void (*make_image) (GstVideoTestSrc *v, GstVideoFrame *frame);
 
   /* temporary AYUV/ARGB scanline */
