@@ -35,13 +35,11 @@
 #define TO_16(x) (((x)<<8) | (x))
 
 static unsigned char
-random_char (void)
+random_char (guint * state)
 {
-  static unsigned int state;
-
-  state *= 1103515245;
-  state += 12345;
-  return (state >> 16) & 0xff;
+  *state *= 1103515245;
+  *state += 12345;
+  return (*state >> 16) & 0xff;
 }
 
 enum
@@ -413,7 +411,7 @@ gst_video_test_src_smpte (GstVideoTestSrc * v, GstVideoFrame * frame)
       p->color = &color;
 
       for (i = x1; i < w; i++) {
-        int y = random_char ();
+        int y = random_char (&v->random_state);
         p->tmpline_u8[i] = y;
       }
       videotestsrc_blend_line (v, p->tmpline + x1 * 4, p->tmpline_u8 + x1,
@@ -517,7 +515,7 @@ gst_video_test_src_snow (GstVideoTestSrc * v, GstVideoFrame * frame)
 
   for (j = 0; j < h; j++) {
     for (i = 0; i < w; i++) {
-      int y = random_char ();
+      int y = random_char (&v->random_state);
       p->tmpline_u8[i] = y;
     }
     videotestsrc_blend_line (v, p->tmpline, p->tmpline_u8,
@@ -1186,8 +1184,6 @@ convert_hline_generic (paintinfo * p, GstVideoFrame * frame, int y)
 
     for (i = 0; i < n_lines; i++) {
       idx = CLAMP (y + i + offset, 0, height - 1);
-
-      GST_DEBUG ("line %d, %d, idx %d", i, y + i + offset, idx);
       lines[i] = p->lines[idx % n_lines];
     }
 
@@ -1198,7 +1194,6 @@ convert_hline_generic (paintinfo * p, GstVideoFrame * frame, int y)
       idx = y + i + offset;
       if (idx > height - 1)
         break;
-      GST_DEBUG ("pack line %d", idx);
       finfo->pack_func (finfo, GST_VIDEO_PACK_FLAG_NONE,
           lines[i], 0, frame->data, frame->info.stride,
           frame->info.chroma_site, idx, width);
