@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <unistd.h>
@@ -40,6 +40,10 @@ static GstPad *mysrcpad, *mysinkpad;
     "framerate = (fraction) 25/1 , "    \
     "format = (string) I420"
 
+#define VIDEO_CAPS_FORCE_VARIABLE_FRAMERATE_STRING \
+    "video/x-raw, "                 \
+    "framerate = (fraction) 0/1"
+
 #define VIDEO_CAPS_NO_FRAMERATE_STRING  \
     "video/x-raw, "                 \
     "width = (int) 320, "               \
@@ -77,6 +81,13 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     GST_STATIC_CAPS (VIDEO_CAPS_TEMPLATE_STRING)
     );
 
+static GstStaticPadTemplate force_variable_rate_template =
+GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS (VIDEO_CAPS_FORCE_VARIABLE_FRAMERATE_STRING)
+    );
+
 static void
 assert_videorate_stats (GstElement * videorate, const gchar * reason,
     guint64 xin, guint64 xout, guint64 xdropped, guint64 xduplicated)
@@ -165,7 +176,7 @@ GST_START_TEST (test_one)
   inbuffer = gst_buffer_new_and_alloc (4);
   gst_buffer_memset (inbuffer, 0, 0, 4);
   caps = gst_caps_from_string (VIDEO_CAPS_STRING);
-  gst_pad_set_caps (mysrcpad, caps);
+  gst_check_setup_events (mysrcpad, videorate, caps, GST_FORMAT_TIME);
   GST_BUFFER_TIMESTAMP (inbuffer) = 0;
   gst_caps_unref (caps);
   ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
@@ -207,7 +218,7 @@ GST_START_TEST (test_more)
   GST_BUFFER_OFFSET_END (first) = g_rand_int (rand);
   gst_buffer_memset (first, 0, 1, 4);
   caps = gst_caps_from_string (VIDEO_CAPS_STRING);
-  gst_pad_set_caps (mysrcpad, caps);
+  gst_check_setup_events (mysrcpad, videorate, caps, GST_FORMAT_TIME);
   gst_caps_unref (caps);
   ASSERT_BUFFER_REFCOUNT (first, "first", 1);
   gst_buffer_ref (first);
@@ -325,7 +336,7 @@ GST_START_TEST (test_wrong_order_from_zero)
   GST_BUFFER_TIMESTAMP (first) = GST_SECOND;
   gst_buffer_memset (first, 0, 0, 4);
   caps = gst_caps_from_string (VIDEO_CAPS_STRING);
-  gst_pad_set_caps (mysrcpad, caps);
+  gst_check_setup_events (mysrcpad, videorate, caps, GST_FORMAT_TIME);
   gst_caps_unref (caps);
   ASSERT_BUFFER_REFCOUNT (first, "first", 1);
   gst_buffer_ref (first);
@@ -408,7 +419,7 @@ GST_START_TEST (test_wrong_order)
   GST_BUFFER_TIMESTAMP (first) = 0;
   gst_buffer_memset (first, 0, 0, 4);
   caps = gst_caps_from_string (VIDEO_CAPS_STRING);
-  gst_pad_set_caps (mysrcpad, caps);
+  gst_check_setup_events (mysrcpad, videorate, caps, GST_FORMAT_TIME);
   gst_caps_unref (caps);
   ASSERT_BUFFER_REFCOUNT (first, "first", 1);
   gst_buffer_ref (first);
@@ -506,7 +517,7 @@ GST_START_TEST (test_no_framerate)
   inbuffer = gst_buffer_new_and_alloc (4);
   gst_buffer_memset (inbuffer, 0, 0, 4);
   caps = gst_caps_from_string (VIDEO_CAPS_NO_FRAMERATE_STRING);
-  gst_pad_set_caps (mysrcpad, caps);
+  gst_check_setup_events (mysrcpad, videorate, caps, GST_FORMAT_TIME);
   gst_caps_unref (caps);
   ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
 
@@ -525,9 +536,9 @@ GST_START_TEST (test_no_framerate)
 
 GST_END_TEST;
 
-/* This test outputs 2 buffers of same dimensions (320x240), then 1 buffer of 
- * differing dimensions (240x120), and then another buffer of previous 
- * dimensions (320x240) and checks that the 3 buffers output as a result have 
+/* This test outputs 2 buffers of same dimensions (320x240), then 1 buffer of
+ * differing dimensions (240x120), and then another buffer of previous
+ * dimensions (320x240) and checks that the 3 buffers output as a result have
  * correct caps (first 2 with 320x240 and 3rd with 240x120).
  */
 GST_START_TEST (test_changing_size)
@@ -539,24 +550,18 @@ GST_START_TEST (test_changing_size)
   GstBuffer *fourth;
   GstBuffer *fifth;
   GstBuffer *outbuf;
-  GstEvent *newsegment;
   GstCaps *caps, *caps_newsize;
-  GstSegment segment;
 
   videorate = setup_videorate ();
   fail_unless (gst_element_set_state (videorate,
           GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
       "could not set to playing");
 
-  gst_segment_init (&segment, GST_FORMAT_TIME);
-  newsegment = gst_event_new_segment (&segment);
-  fail_unless (gst_pad_push_event (mysrcpad, newsegment) == TRUE);
-
   first = gst_buffer_new_and_alloc (4);
   gst_buffer_memset (first, 0, 0, 4);
   caps = gst_caps_from_string (VIDEO_CAPS_STRING);
   GST_BUFFER_TIMESTAMP (first) = 0;
-  gst_pad_set_caps (mysrcpad, caps);
+  gst_check_setup_events (mysrcpad, videorate, caps, GST_FORMAT_TIME);
 
   GST_DEBUG ("pushing first buffer");
   fail_unless (gst_pad_push (mysrcpad, first) == GST_FLOW_OK);
@@ -631,7 +636,7 @@ GST_START_TEST (test_non_ok_flow)
   buf = gst_buffer_new_and_alloc (4);
   gst_buffer_memset (buf, 0, 0, 4);
   caps = gst_caps_from_string (VIDEO_CAPS_STRING);
-  gst_pad_set_caps (mysrcpad, caps);
+  gst_check_setup_events (mysrcpad, videorate, caps, GST_FORMAT_TIME);
   gst_caps_unref (caps);
   ASSERT_BUFFER_REFCOUNT (buf, "inbuffer", 1);
 
@@ -973,6 +978,8 @@ static void
 check_caps_identical (GstCaps * a, GstCaps * b, const char *name)
 {
   int i;
+  gchar *caps_str_a;
+  gchar *caps_str_b;
 
   if (gst_caps_get_size (a) != gst_caps_get_size (b))
     goto fail;
@@ -990,8 +997,11 @@ check_caps_identical (GstCaps * a, GstCaps * b, const char *name)
   return;
 
 fail:
-  fail ("%s caps (%s) is not equal to caps (%s)",
-      name, gst_caps_to_string (a), gst_caps_to_string (b));
+  caps_str_a = gst_caps_to_string (a);
+  caps_str_b = gst_caps_to_string (b);
+  fail ("%s caps (%s) is not equal to caps (%s)", name, caps_str_a, caps_str_b);
+  g_free (caps_str_a);
+  g_free (caps_str_b);
 }
 
 static void
@@ -1021,8 +1031,10 @@ GST_START_TEST (test_caps_negotiation)
   videorate = setup_videorate_full (&srctemplate, &sinktemplate);
 
   caps = gst_caps_from_string (test->caps);
+
   g_object_set_data_full (G_OBJECT (mysrcpad), "caps",
       gst_caps_ref (caps), (GDestroyNotify) gst_caps_unref);
+
   g_object_set_data_full (G_OBJECT (mysinkpad), "caps",
       gst_caps_ref (caps), (GDestroyNotify) gst_caps_unref);
   gst_caps_unref (caps);
@@ -1037,7 +1049,348 @@ GST_START_TEST (test_caps_negotiation)
   check_peer_caps (mysrcpad, test->expected_sink_caps, "sink");
   check_peer_caps (mysinkpad, test->expected_src_caps, "src");
 
-  gst_object_unref (videorate);
+  cleanup_videorate (videorate);
+}
+
+GST_END_TEST;
+
+static void
+videorate_send_buffers (GstElement * videorate,
+    const gchar * pre_push_caps, const gchar * post_push_caps)
+{
+  GstCaps *caps, *expected_caps;
+  GstBuffer *first;
+  GstBuffer *second;
+  GstBuffer *third;
+
+  caps = gst_pad_get_current_caps (mysinkpad);
+  expected_caps = gst_caps_from_string (pre_push_caps);
+  gst_check_caps_equal (caps, expected_caps);
+  gst_caps_unref (caps);
+  gst_caps_unref (expected_caps);
+
+  GST_DEBUG ("pushing first buffer");
+  first = gst_buffer_new_and_alloc (4);
+  gst_buffer_memset (first, 0, 0, 4);
+  GST_BUFFER_TIMESTAMP (first) = 0;
+  fail_unless (gst_pad_push (mysrcpad, first) == GST_FLOW_OK);
+
+  /* second buffer */
+  second = gst_buffer_new_and_alloc (4);
+  GST_BUFFER_TIMESTAMP (second) = GST_SECOND / 25;
+  gst_buffer_memset (second, 0, 0, 4);
+
+  fail_unless (gst_pad_push (mysrcpad, second) == GST_FLOW_OK);
+
+  /* third buffer with new size */
+  third = gst_buffer_new_and_alloc (4);
+  GST_BUFFER_TIMESTAMP (third) = 2 * GST_SECOND / 25;
+  gst_buffer_memset (third, 0, 0, 4);
+
+  fail_unless (gst_pad_push (mysrcpad, third) == GST_FLOW_OK);
+
+  caps = gst_pad_get_current_caps (mysinkpad);
+  expected_caps = gst_caps_from_string (post_push_caps);
+  gst_check_caps_equal (caps, expected_caps);
+  gst_caps_unref (caps);
+  gst_caps_unref (expected_caps);
+
+}
+
+GST_START_TEST (test_fixed_framerate)
+{
+  GstElement *videorate;
+  GstCaps *caps;
+
+  /* 1) if upstream caps contain a non-0/1 framerate, we should use that and pass
+   *    it on downstream (if possible; otherwise fixate_to_nearest)
+   */
+  videorate = setup_videorate_full (&srctemplate, &sinktemplate);
+
+  caps = gst_caps_from_string ("video/x-raw,framerate=25/1");
+  ASSERT_SET_STATE (videorate, GST_STATE_PLAYING, GST_STATE_CHANGE_SUCCESS);
+  gst_check_setup_events (mysrcpad, videorate, caps, GST_FORMAT_TIME);
+  gst_caps_unref (caps);
+  videorate_send_buffers (videorate, "video/x-raw,framerate=25/1",
+      "video/x-raw,framerate=25/1");
+  cleanup_videorate (videorate);
+
+  /* 2) if upstream framerate is 0/1 and downstream doesn't force a particular
+   *    framerate, we try to guess based on buffer intervals and use that as output
+   *    framerate */
+  videorate = setup_videorate_full (&srctemplate, &sinktemplate);
+  ASSERT_SET_STATE (videorate, GST_STATE_PLAYING, GST_STATE_CHANGE_SUCCESS);
+  caps = gst_caps_from_string ("video/x-raw,framerate=0/1");
+  gst_check_setup_events (mysrcpad, videorate, caps, GST_FORMAT_TIME);
+  gst_caps_unref (caps);
+  videorate_send_buffers (videorate, "video/x-raw,framerate=0/1",
+      "video/x-raw,framerate=25/1");
+  cleanup_videorate (videorate);
+
+  /* 3) if downstream force variable framerate, do that */
+  videorate =
+      setup_videorate_full (&srctemplate, &force_variable_rate_template);
+  ASSERT_SET_STATE (videorate, GST_STATE_PLAYING, GST_STATE_CHANGE_SUCCESS);
+  caps = gst_caps_from_string ("video/x-raw,framerate=0/1");
+  gst_check_setup_events (mysrcpad, videorate, caps, GST_FORMAT_TIME);
+  gst_caps_unref (caps);
+  videorate_send_buffers (videorate, "video/x-raw,framerate=0/1",
+      "video/x-raw,framerate=0/1");
+  cleanup_videorate (videorate);
+
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_variable_framerate_renegotiation)
+{
+  GstElement *videorate;
+  GstCaps *caps;
+  GstCaps *allowed;
+
+  videorate = setup_videorate_full (&srctemplate, &sinktemplate);
+  ASSERT_SET_STATE (videorate, GST_STATE_PLAYING, GST_STATE_CHANGE_SUCCESS);
+  caps = gst_caps_from_string ("video/x-raw,framerate=0/1");
+  gst_check_setup_events (mysrcpad, videorate, caps, GST_FORMAT_TIME);
+  videorate_send_buffers (videorate, "video/x-raw,framerate=0/1",
+      "video/x-raw,framerate=25/1");
+
+  /* framerate=0/1 must still be allowed to be configured on
+   * the upstream side of videorate */
+  allowed = gst_pad_get_allowed_caps (mysrcpad);
+  fail_unless (gst_caps_is_subset (caps, allowed) == TRUE);
+
+  gst_caps_unref (allowed);
+  gst_caps_unref (caps);
+  cleanup_videorate (videorate);
+}
+
+GST_END_TEST;
+
+/* Rate tests info */
+typedef struct
+{
+  gdouble rate;
+  guint64 expected_in, expected_out, expected_drop, expected_dup;
+  gint current_buf;
+  GstClockTime expected_ts;
+} RateInfo;
+
+static RateInfo rate_tests[] = {
+  {
+        .rate = 1.0,
+        .expected_in = 34,
+        .expected_out = 25,
+        .expected_drop = 8,
+        .expected_dup = 0,
+        .current_buf = 0,
+      .expected_ts = 0},
+  {
+        .rate = 0.5,
+        .expected_in = 34,
+        .expected_out = 50,
+        .expected_drop = 0,
+        .expected_dup = 17,
+        .current_buf = 0,
+      .expected_ts = 0},
+  {
+        .rate = 2.0,
+        .expected_in = 34,
+        .expected_out = 13,
+        .expected_drop = 20,
+        .expected_dup = 0,
+        .current_buf = 0,
+      .expected_ts = 0},
+};
+
+static GstPadProbeReturn
+listen_outbuffer_ts (GstPad * pad, GstPadProbeInfo * info, gpointer user_data)
+{
+  GstBuffer *buffer;
+  guint64 buf_ts;
+  RateInfo *test = (RateInfo *) user_data;
+
+  buffer = GST_PAD_PROBE_INFO_BUFFER (info);
+  buf_ts = GST_BUFFER_TIMESTAMP (buffer);
+
+  GST_DEBUG ("Probed %d outbuf. ts : %" GST_TIME_FORMAT
+      ", expected : %" GST_TIME_FORMAT, test->current_buf,
+      GST_TIME_ARGS (buf_ts), GST_TIME_ARGS (test->expected_ts));
+  fail_unless_equals_uint64 (buf_ts, test->expected_ts);
+
+  /* Next expected timestamp with fps 25/1 */
+  test->expected_ts += 40000000;
+  test->current_buf += 1;
+
+  fail_if (test->current_buf > test->expected_out);
+
+  return GST_PAD_PROBE_OK;
+}
+
+GST_START_TEST (test_rate)
+{
+  GstElement *videorate;
+  RateInfo *test = &rate_tests[__i__];
+  GstClockTime ts;
+  GstBuffer *buf;
+  GstCaps *caps;
+  gulong probe;
+
+  videorate = setup_videorate ();
+  fail_unless (gst_element_set_state (videorate,
+          GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
+      "could not set to playing");
+  probe = gst_pad_add_probe (mysinkpad, GST_PAD_PROBE_TYPE_BUFFER,
+      (GstPadProbeCallback) listen_outbuffer_ts, test, NULL);
+
+  buf = gst_buffer_new_and_alloc (4);
+  gst_buffer_memset (buf, 0, 0, 4);
+  caps = gst_caps_from_string (VIDEO_CAPS_STRING);
+  gst_check_setup_events (mysrcpad, videorate, caps, GST_FORMAT_TIME);
+  gst_caps_unref (caps);
+  ASSERT_BUFFER_REFCOUNT (buf, "inbuffer", 1);
+
+  /* Setting rate */
+  g_object_set (videorate, "rate", test->rate, NULL);
+
+  /* Push 1 second of buffers */
+  for (ts = 0; ts < 1 * GST_SECOND; ts += GST_SECOND / 33) {
+    GstBuffer *inbuf;
+
+    inbuf = gst_buffer_copy (buf);
+    GST_BUFFER_TIMESTAMP (inbuf) = ts;
+
+    fail_unless_equals_int (gst_pad_push (mysrcpad, inbuf), GST_FLOW_OK);
+  }
+
+  fail_unless_equals_int (g_list_length (buffers), test->expected_out);
+  assert_videorate_stats (videorate, "last buffer", test->expected_in,
+      test->expected_out, test->expected_drop, test->expected_dup);
+
+  /* cleanup */
+  gst_pad_remove_probe (mysinkpad, probe);
+  cleanup_videorate (videorate);
+  gst_buffer_unref (buf);
+}
+
+GST_END_TEST;
+
+/* Probing the pad to force a fake upstream duration */
+static GstPadProbeReturn
+listen_sink_query_duration (GstPad * pad, GstPadProbeInfo * info,
+    gpointer user_data)
+{
+  GstQuery *query;
+  gint64 *duration = (gint64 *) user_data;
+
+  query = gst_pad_probe_info_get_query (info);
+
+  if (GST_QUERY_TYPE (query) == GST_QUERY_DURATION) {
+    gst_query_set_duration (query, GST_FORMAT_TIME, *duration);
+  }
+  return GST_PAD_PROBE_OK;
+}
+
+GST_START_TEST (test_query_duration)
+{
+  GstElement *videorate;
+  gulong probe_sink;
+  gint64 duration;
+  GstQuery *query;
+
+  videorate = setup_videorate ();
+  fail_unless (gst_element_set_state (videorate,
+          GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
+      "could not set to playing");
+  probe_sink =
+      gst_pad_add_probe (mysrcpad,
+      GST_PAD_PROBE_TYPE_QUERY_DOWNSTREAM | GST_PAD_PROBE_TYPE_PUSH,
+      (GstPadProbeCallback) listen_sink_query_duration, &duration, NULL);
+
+  query = gst_query_new_duration (GST_FORMAT_TIME);
+  duration = GST_CLOCK_TIME_NONE;
+  gst_pad_peer_query (mysrcpad, query);
+  gst_query_parse_duration (query, NULL, &duration);
+  fail_unless_equals_uint64 (duration, GST_CLOCK_TIME_NONE);
+
+  /* Setting fake upstream duration to 1 second */
+  duration = GST_SECOND;
+
+  /* Setting rate to 2.0 */
+  g_object_set (videorate, "rate", 2.0, NULL);
+
+  gst_pad_peer_query (mysrcpad, query);
+  gst_query_parse_duration (query, NULL, &duration);
+  fail_unless_equals_uint64 (duration, 0.5 * GST_SECOND);
+
+  /* cleanup */
+  gst_query_unref (query);
+  gst_pad_remove_probe (mysrcpad, probe_sink);
+  cleanup_videorate (videorate);
+}
+
+GST_END_TEST;
+
+/* Position tests info */
+typedef struct
+{
+  gdouble rate;
+} PositionInfo;
+
+static PositionInfo position_tests[] = {
+  {
+      .rate = 1.0},
+  {
+      .rate = 0.5},
+  {
+      .rate = 2.0},
+  {
+      .rate = 1.7},
+};
+
+GST_START_TEST (test_query_position)
+{
+  GstElement *videorate;
+  PositionInfo *test = &position_tests[__i__];
+  GstClockTime ts;
+  GstBuffer *buf;
+  GstCaps *caps;
+  gint64 position, expected_position = 0;
+
+  videorate = setup_videorate ();
+  fail_unless (gst_element_set_state (videorate,
+          GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
+      "could not set to playing");
+
+  buf = gst_buffer_new_and_alloc (4);
+  gst_buffer_memset (buf, 0, 0, 4);
+  caps = gst_caps_from_string (VIDEO_CAPS_STRING);
+  gst_check_setup_events (mysrcpad, videorate, caps, GST_FORMAT_TIME);
+  gst_caps_unref (caps);
+  ASSERT_BUFFER_REFCOUNT (buf, "inbuffer", 1);
+
+  /* Push a few buffers */
+  g_object_set (videorate, "rate", test->rate, NULL);
+  for (ts = 0; ts < GST_SECOND; ts += GST_SECOND / 20) {
+    GstBuffer *inbuf;
+
+    inbuf = gst_buffer_copy (buf);
+    GST_BUFFER_TIMESTAMP (inbuf) = ts;
+
+    fail_unless_equals_int (gst_pad_push (mysrcpad, inbuf), GST_FLOW_OK);
+
+    expected_position = ts / test->rate;
+    gst_element_query_position (videorate, GST_FORMAT_TIME, &position);
+    GST_DEBUG_OBJECT (NULL,
+        "pushed buffer %" GST_TIME_FORMAT ", queried pos: %" GST_TIME_FORMAT
+        ", expected pos: %" GST_TIME_FORMAT, GST_TIME_ARGS (ts),
+        GST_TIME_ARGS (position), GST_TIME_ARGS (expected_position));
+    fail_unless_equals_uint64 (position, expected_position);
+  }
+
+  /* cleanup */
+  cleanup_videorate (videorate);
+  gst_buffer_unref (buf);
 }
 
 GST_END_TEST;
@@ -1060,6 +1413,12 @@ videorate_suite (void)
   tcase_add_test (tc_chain, test_selected_caps);
   tcase_add_loop_test (tc_chain, test_caps_negotiation,
       0, G_N_ELEMENTS (caps_negotiation_tests));
+  tcase_add_test (tc_chain, test_fixed_framerate);
+  tcase_add_test (tc_chain, test_variable_framerate_renegotiation);
+  tcase_add_loop_test (tc_chain, test_rate, 0, G_N_ELEMENTS (rate_tests));
+  tcase_add_test (tc_chain, test_query_duration);
+  tcase_add_loop_test (tc_chain, test_query_position, 0,
+      G_N_ELEMENTS (position_tests));
 
   return s;
 }
